@@ -83,7 +83,7 @@ def upload(destination, file, progress_bar=False, filename='features_backup.rcdb
             return requests.post(
                 url, data=monitor, headers=headers
             )
-    elif provider == 'azure-blob':
+    elif provider == 'azure-blob' or provider == 'aws-s3-put':
         return requests.put(
             url,
             data=file,
