@@ -49,8 +49,8 @@ class AdminProvider(ReboticsBaseProvider):
             'host': host
         })
 
-    def set_retailer_identifier(self, retailer_id, retailer_secret_key):
-        self.headers['x-retailer-id'] = retailer_id
+    def set_retailer_identifier(self, retailer_codename, retailer_secret_key):
+        self.headers['x-retailer-codename'] = retailer_codename
         self.headers['x-retailer-secret-key'] = retailer_secret_key
 
     @remote_service('/api/token-auth/', raw=True)
