@@ -1,9 +1,11 @@
 import json
 import logging
 import time
+import typing
 from collections import OrderedDict
 from copy import copy
 from pprint import pformat
+from uuid import uuid4
 
 import requests
 from six import wraps
@@ -49,8 +51,9 @@ def flatten_nested_object(obj, parent_key=''):
     return result
 
 
+# noinspection PyMethodMayBeStatic
 class ProviderRequestProxy(object):
-    def __init__(self, url, headers, timeout, retries=MAX_RETRIES, host=None, retry_delay=5, **kwargs):
+    def __init__(self, *, url, headers, timeout, retries=MAX_RETRIES, host=None, retry_delay=5, **kwargs):
         self.url = url
         self.headers = headers
         self.timeout = timeout
@@ -143,6 +146,23 @@ class ProviderRequestProxy(object):
             except Exception as exc:
                 logger.debug('Failed to put file pointer to the start of the file with exc: %s', exc, exc_info=exc)
 
+    def raise_if_error_status_code(self, response: requests.Response):
+        if 400 <= response.status_code < 500:
+            http_error_msg = '%s Client Error: %s for url: %s %s' % \
+                             (response.status_code,
+                              response.reason,
+                              response.url,
+                              response.content)
+            raise ProviderHTTPClientException(http_error_msg, response=response)
+        elif response.status_code in [502, 503]:
+            raise RetryableException(
+                'Deployment might be in progress, or server is overloaded')
+        elif 500 <= response.status_code < 600:
+            raise ProviderHTTPClientException('%d Server error. %s ' % (
+                response.status_code,
+                response.reason
+            ), response=response)
+
     def request_with_retry(self, method, **request_arguments):
         request_arguments = self.augment_request_arguments(**request_arguments)
         retries = 0
@@ -152,20 +172,7 @@ class ProviderRequestProxy(object):
             try:
                 self.set_file_caret_to_start(**request_arguments)
                 response = getattr(requests, method)(**request_arguments)
-                if 400 <= response.status_code < 500:
-                    http_error_msg = '%s Client Error: %s for url: %s %s' % \
-                                     (response.status_code,
-                                      response.reason,
-                                      response.url,
-                                      response.content)
-                    raise ProviderHTTPClientException(http_error_msg, response=response)
-                elif response.status_code in [502, 503]:
-                    raise RetryableException('Deployment might be in progress, or server is overloaded')
-                elif 500 <= response.status_code < 600:
-                    raise ProviderHTTPClientException('%d Server error. %s ' % (
-                        response.status_code,
-                        response.reason
-                    ), response=response)
+                self.raise_if_error_status_code(response)
 
             except (requests.Timeout, RetryableException) as exc:
                 logger.debug('Retry request %s %s', method, request_arguments['url'])
@@ -184,7 +191,58 @@ class ProviderRequestProxy(object):
         return self.format_return_data(response)
 
 
-def remote_service(path, headers=None, timeout=None, json=True, raw=False):
+class UniqueRequestProxy(ProviderRequestProxy):
+    ENABLED_METHODS = ('post', 'put')
+
+    def __init__(self,
+                 *,
+                 request_uid_header_name: str,
+                 request_uid_value: typing.Callable[..., str] = None,
+                 **kwargs):
+        """
+        :param request_uid_header_name: HTTP header name that will contain the
+        request UUID
+        :param request_uid_value: callable that will receive the request context and
+        must return the string with UUID value; if callable is not provided or
+        returned None, UUID will be generated automatically for every request
+        """
+        super(UniqueRequestProxy, self).__init__(**kwargs)
+
+        self.header_name = request_uid_header_name
+        self.uid_value = request_uid_value
+
+    def inject_unique_request_header(self, **request_arguments):
+        uuid_value = None
+        if callable(self.uid_value):
+            uuid_value = self.uid_value(**request_arguments)
+        if uuid_value is None:
+            uuid_value = str(uuid4())
+
+        self.headers[self.header_name] = uuid_value
+
+    def request_with_retry(self, method, **request_arguments):
+        if method in self.ENABLED_METHODS:
+            self.inject_unique_request_header(**request_arguments)
+
+        return super(UniqueRequestProxy, self).request_with_retry(
+            method,
+            **request_arguments
+        )
+
+    def raise_if_error_status_code(self, response: requests.Response):
+        # Handle "Another request is already processing" error with retry
+        if response.status_code == 424:
+            try:
+                json_resp = response.json()
+                error_desc = json_resp['error']
+            except (ValueError, TypeError, KeyError):
+                error_desc = 'Another request is already processing'
+            raise RetryableException(error_desc)
+
+        return super(UniqueRequestProxy, self).raise_if_error_status_code(response)
+
+
+def remote_service(path, headers=None, timeout=None, json=True, raw=False, **options):
     if headers is None:
         headers = {}
     assert isinstance(headers, dict), 'Headers should be a dict!'
@@ -205,15 +263,18 @@ def remote_service(path, headers=None, timeout=None, json=True, raw=False):
 
             url = provider.build_url(path)
             raw_override = kwargs.get('raw')
+            proxy_opts = copy(provider.proxy_config)
+            proxy_opts.update(options)
 
-            session = ProviderRequestProxy(
-                url,
-                headers,
+            session = provider.proxy_class(
+                url=url,
+                headers=headers,
                 timeout=timeout or provider.timeout,
                 retries=provider.retries,
                 json=json,
                 raw=raw_override or raw,
-                host=provider.host
+                host=provider.host,
+                **proxy_opts
             )
 
             provider.session = session
@@ -233,6 +294,8 @@ def remote_service(path, headers=None, timeout=None, json=True, raw=False):
 class ReboticsBaseProvider(object):
     retries = MAX_RETRIES
     timeout = DEFAULT_TIMEOUT
+    proxy_class = ProviderRequestProxy
+    proxy_config = {}  # additional parameters that will be passed to proxy_cls' init
 
     def __init__(self, host, **kwargs):
         self.host = host
