@@ -3,12 +3,17 @@ import os
 import pathlib
 import re
 from timeit import default_timer
+from urllib.parse import uses_relative, uses_netloc, uses_params
 
 import requests
 from six.moves.urllib_parse import urlparse
+from urllib3.util import parse_url
 
 from rebotics_sdk.advanced import remote_loaders
 
+_VALID_URLS = set(uses_relative + uses_netloc + uses_params)
+_VALID_URLS.discard("")
+
 
 def mkdir_p(path):
     try:
@@ -20,6 +25,24 @@ def mkdir_p(path):
             raise
 
 
+def is_url(url) -> bool:
+    """
+    Check to see if a URL has a valid protocol.
+
+    Parameters
+    ----------
+    url : str or unicode
+
+    Returns
+    -------
+    isurl : bool
+        If `url` has a valid protocol return True otherwise False.
+    """
+    if not isinstance(url, str):
+        return False
+    return parse_url(url).scheme in _VALID_URLS
+
+
 def get_filename_from_url(url):
     return urlparse(url).path.split('/')[-1]
 
